'**************************************************************************************************
' File:			HOME.PRG
' Purpose:		Home all the drives
' Version:		1.00
' Author:		Nir Geller
' Global Variables Used: num_Of_Motion_Drives as long  - Number of ECat MOTION DRIVES found by EC_SETUP.PRG
'             	AxesList[64] - An array that holds all the axes in the system. Defined in CONFIG.PRG
' History:		9.11.14 - Created
' Description:	This program example iterates over the axes and calls homing procedure.
'				The homing parameters setting and the homing procedure is CDHD specific
'**************************************************************************************************

'                             CDHD CANOPEN HOMING EXAMPLE

'**************										Explanation										**************

'				Before starting the homing procedure the user must set homing parameters in the drive.
'				These parameters must be in "drive units", therefore, the desired homing velocity is translated to
'				drive units by multiplying it by the velocity factor and by a factor of 1000 in order to scale from
'				seconds to milliseconds. The same is done for homing acceleration.

'*****************************************************************************************************************



Program
  dim Home_method as long
	dim dHomingFactorFast	as double = 0.5
	dim dHomingFactorslow	as double = 0.25
	dim dHomingFactorAcc	as double = 0.1
	dim dSec2msec			as double = 1000  '	this is used to convert from typical user time-units [msec] to typical drive time-units [sec]
	dim dHomingVelocity		as double = 0
	dim dHomingAcceleration	as double = 0
	dim i as long = 0


  dim HOMING_INPUT_SHOULD_BE_SET as long = 0
  dim POSITIVE_LS_INPUT_SHOULD_BE_SET as long =0
  dim NEGATIVE_LS_INPUT_SHOULD_BE_SET as long =0

  

  Home_method = 34 


    i=1 ' for A1 axis , containing in AxesList[1] 

		attach AxesList[i]

			' **** Set homing parameters ****
			' set homing method = 34 (pulse index, positive direction)
			call CAN_SLAVE_SET_HOMING_PARAMETERS(AxesList[i], CAN_HOMING_METHOD, Home_method)

			' set homing offset
			call CAN_SLAVE_SET_HOMING_PARAMETERS(AxesList[i], CAN_HOMING_OFFSET, 0)

			' set FAST homing speed = 10
			dHomingVelocity = (AxesList[i].VCruise * AxesList[i].VFac * dSec2msec) * dHomingFactorFast
			call CAN_SLAVE_SET_HOMING_PARAMETERS(AxesList[i], CAN_FAST_HOMING_SPEED, dHomingVelocity)

			' set SLOW homing speed = 2
			dHomingVelocity = (AxesList[i].VCruise * AxesList[i].VFac * dSec2msec) * dHomingFactorSlow
			call CAN_SLAVE_SET_HOMING_PARAMETERS(AxesList[i], CAN_SLOW_HOMING_SPEED, dHomingVelocity)

			' set homing acceleration/deceleration
			dHomingAcceleration = (AxesList[i].amax * AxesList[i].afac * dSec2msec^2) * dHomingFactorAcc
			call CAN_SLAVE_SET_HOMING_PARAMETERS(AxesList[i], CAN_HOMING_ACCELERATION, dHomingAcceleration)


     '////////////////////////////////////////////////////////////////////////////////
     ' BE SURE THAT THE DRIVE INPUTS MODE , WHAT NEED FOR CHOSEN HOME TYPE , IS SET !!!
     '////////////////////////////////////////////////////////////////////////////////////////
      select case  Home_method
        case 1 , 17
         ' ?"check negative limit switch"
          if CDHD_IS_ANY_INPUT_NEG_LS_MODE(AxesList[i].dadd) = 0 then
            NEGATIVE_LS_INPUT_SHOULD_BE_SET = 1
          end if
          
        case 2 , 18
        '  ?"check positive limit switch"
           if CDHD_IS_ANY_INPUT_POS_LS_MODE(AxesList[i].dadd) = 0 then
             POSITIVE_LS_INPUT_SHOULD_BE_SET = 1            
           end if

        case 3 to 6 
          '?"check home switch"
          if CDHD_IS_ANY_INPUT_HOMING_MODE(AxesList[i].dadd) = 0 then
            HOMING_INPUT_SHOULD_BE_SET = 1           
          end if

        case 7 to 10
         '?" check positive limit switch + home switch"
         if CDHD_IS_ANY_INPUT_POS_LS_MODE(AxesList[i].dadd) = 0 then
            POSITIVE_LS_INPUT_SHOULD_BE_SET = 1        
         end if

         if CDHD_IS_ANY_INPUT_HOMING_MODE(AxesList[i].dadd) = 0 then
            HOMING_INPUT_SHOULD_BE_SET = 1          
         end if

        case 11 to 14
          '?"check negative limit switch + home switch"
          if CDHD_IS_ANY_INPUT_NEG_LS_MODE(AxesList[i].dadd) = 0 then
            NEGATIVE_LS_INPUT_SHOULD_BE_SET = 1
          end if

         if CDHD_IS_ANY_INPUT_HOMING_MODE(AxesList[i].dadd) = 0 then
            HOMING_INPUT_SHOULD_BE_SET = 1          
         end if

        case 19 to 22
         ' ?"check home switch"
          if CDHD_IS_ANY_INPUT_HOMING_MODE(AxesList[i].dadd) = 0 then
            HOMING_INPUT_SHOULD_BE_SET = 1          
          end if

         case 23 to 26
         ' ?"check positive limit switch+ home switch"
           if CDHD_IS_ANY_INPUT_POS_LS_MODE(AxesList[i].dadd) = 0 then
              POSITIVE_LS_INPUT_SHOULD_BE_SET = 1        
           end if

           if CDHD_IS_ANY_INPUT_HOMING_MODE(AxesList[i].dadd) = 0 then
              HOMING_INPUT_SHOULD_BE_SET = 1        
           end if

        case 27 to 30
        '  ?"check negative limit switch + home switch"
          if CDHD_IS_ANY_INPUT_NEG_LS_MODE(AxesList[i].dadd) = 0 then
            NEGATIVE_LS_INPUT_SHOULD_BE_SET = 1
          end if

           if CDHD_IS_ANY_INPUT_HOMING_MODE(AxesList[i].dadd) = 0 then
               HOMING_INPUT_SHOULD_BE_SET = 1           
           end if

         case else
        ' ?"case else"

      end select

      if  HOMING_INPUT_SHOULD_BE_SET = 1 then
        call CDHD_DIN_HOMING_MODE_SET(AxesList[i].dadd,2)     ' set digital input 2 as homing switch , for example
      end if

      if POSITIVE_LS_INPUT_SHOULD_BE_SET = 1 then
        call CDHD_DIN_POSITIVE_LS_MODE_SET(AxesList[i].dadd,3) ' set digital input 3 as positive limit switch , for example   
      end if

      if NEGATIVE_LS_INPUT_SHOULD_BE_SET = 1 then
        call CDHD_DIN_NEGATIVE_LS_MODE_SET(AxesList[i].dadd,4) ' set digital input 4 as negative limit switch , for example
      end if
    '////////////////////////////////////////////////////////////////////////


		'	call CAN_SLAVE_HOME(AxesList[i], 120000)  'For CDHD only (uses the manufacturer object 0X2090 home status )  Homing timeout: 120,000 msec
      call CAN_SLAVE_HOME_DS402(AxesList[i], 120000) 'Generic function (uses the standad object 0x6041 drive status )  Homing timeout: 120,000 msec

			AxesList[i].en = 0

		detach AxesList[i]
    ?"homing finished , home type = ";Home_method



End Program

'**************************************************************
'     CDHD_DIN_IS_HOMING_MODE
'			the function checks  is the specified CDHD digital input set as Homing Switch
'		  returns one  in positive case or zero in negative case
'**************************************************************
function CDHD_DIN_IS_HOMING_MODE( byval driveAddr as long , byval DInput as long) as long
  call can_sdo_write(driveAddr,0x20e0,1,16,DInput)

  if can_sdo_read(driveAddr,0x20e0,2) = 8 then
    CDHD_DIN_IS_HOMING_MODE = 1
    else
      CDHD_DIN_IS_HOMING_MODE = 0
  end if
  
end function

'**************************************************************
'     CDHD_DIN_IS_POSITIVE_LS_MODE
'			the function checks  is the specified CDHD digital input set as Positive Limit Switch
'		  returns one  in positive case or zero in negative case
'**************************************************************
function CDHD_DIN_IS_POSITIVE_LS_MODE( byval driveAddr as long , byval DInput as long) as long
  call can_sdo_write(driveAddr,0x20e0,1,16,DInput)

  if can_sdo_read(driveAddr,0x20e0,2) = 5 then
    CDHD_DIN_IS_POSITIVE_LS_MODE = 1
    else
      CDHD_DIN_IS_POSITIVE_LS_MODE = 0
  end if
  
end function


'**************************************************************
'     CDHD_DIN_IS_NEGATIVE_LS_MODE
'			the function checks  is the specified CDHD digital input set as Negative Limit Switch
'		  returns one  in positive case or zero in negative case
'**************************************************************
function CDHD_DIN_IS_NEGATIVE_LS_MODE( byval driveAddr as long , byval DInput as long) as long
  call can_sdo_write(driveAddr,0x20e0,1,16,DInput)

  if can_sdo_read(driveAddr,0x20e0,2) = 5 then
    CDHD_DIN_IS_NEGATIVE_LS_MODE = 1
    else
      CDHD_DIN_IS_NEGATIVE_LS_MODE = 0
  end if
  
end function


'**************************************************************
'     CDHD_DIN_HOMING_MODE_SET
'     the subroutine set the specified CDHD digital input set as Homing Switch
'		  
'**************************************************************
sub CDHD_DIN_HOMING_MODE_SET( byval driveAddr as long , byval DInput as long) 
  call can_sdo_write(driveAddr,0x20e0,1,16,DInput)

  call can_sdo_write(driveAddr,0x20e0,2,16,8)
end sub

'**************************************************************
'     CDHD_DIN_POSITIVE_LS_MODE_SET
'     the subroutine set the specified CDHD digital input set as Positive Limit Switch
'		  
'**************************************************************
sub CDHD_DIN_POSITIVE_LS_MODE_SET(byval driveAddr as long , byval DInput as long)
  call can_sdo_write(driveAddr,0x20e0,1,16,DInput)

  call can_sdo_write(driveAddr,0x20e0,2,16,5)  
end sub


'**************************************************************
'     CDHD_DIN_NEGATIVE_LS_MODE_SET
'     the subroutine set the specified CDHD digital input set as Negative Limit Switch
'		  
'**************************************************************
sub CDHD_DIN_NEGATIVE_LS_MODE_SET( byval driveAddr as long , byval DInput as long)
  call can_sdo_write(driveAddr,0x20e0,1,16,DInput)

  call can_sdo_write(driveAddr,0x20e0,2,16,6)  
end sub
  


'**************************************************************
'     CDHD_IS_ANY_INPUT_HOMING_MODE
'			the function checks  is any CDHD digital input set as Homing Switch
'		  returns the number of input or zero in negative case
'**************************************************************
function CDHD_IS_ANY_INPUT_HOMING_MODE( byval driveAddr as long) as long
  dim i1 as long
  CDHD_IS_ANY_INPUT_HOMING_MODE = 0
  for i1=1 to 11
    if CDHD_DIN_IS_HOMING_MODE(driveAddr, i1) = 1 then
      CDHD_IS_ANY_INPUT_HOMING_MODE = i1
    end if
  next
end function


'**************************************************************
'     CDHD_IS_ANY_INPUT_POS_LS_MODE
'			the function checks  is any CDHD digital input set as Positive Limit Switch
'		  returns the number of input or zero in negative case
'**************************************************************
function CDHD_IS_ANY_INPUT_POS_LS_MODE( byval driveAddr as long) as long
  dim i1 as long
  CDHD_IS_ANY_INPUT_POS_LS_MODE =0
  for i1=1 to 11
    if CDHD_DIN_IS_POSITIVE_LS_MODE(driveAddr, i1) = 1 then
      CDHD_IS_ANY_INPUT_POS_LS_MODE = i1
    end if
  next  
end function



'**************************************************************
'     CDHD_IS_ANY_INPUT_NEG_LS_MODE
'			the function checks  is any CDHD digital input set as Negative Limit Switch
'		  returns the number of input or zero in negative case
'**************************************************************
function CDHD_IS_ANY_INPUT_NEG_LS_MODE( byval driveAddr as long) as long
  dim i1 as long
  CDHD_IS_ANY_INPUT_NEG_LS_MODE=0
  for i1=1 to 11
    if CDHD_DIN_IS_NEGATIVE_LS_MODE(driveAddr, i1) = 1 then
      CDHD_IS_ANY_INPUT_NEG_LS_MODE = i1
    end if
  next  
end function
