'**********************************  EtherCAT Setup Program  **********************************
' File:        EC_SETUP.PRG
' Purpose:     EtherCAT setup example
' Version:     V1.0.0.7
' Author:      Nir Geller
' Description: This script demonstrates how to setup the EtherCAT module, which function/subroutines to invoke,
'              and in what order. 
'
' History:  20.10.2013  V1.0.0.0 - Created
'
'      15.12.2014  V1.0.0.1 - Adapted to comply with new IOs handling
'      in Ethercat.LIB version 1.0.0.2   ,   MC version 0.4.13.2rc1-C2
'
'      28.12.2014  V1.0.0.2 - EC_STARTMASTER waits for hpet calibration. EC_SETUP.PRG no longer does that.
'      Added user functions for PEROP, SAFEOF and OP configuration.
'
'      5.1.2015  V1.0.0.4 - Creation of PDO mapping and bus cycle time setting is moved from EC_SETUP.PRG into the context of EC_STARTMASTER
'
'      26.4.2015  V1.0.0.5 - If EC_STARTMASTER throws a PLL error, EC_SETUP.PRG is restarted up to 5 times retrying to finish successfully
'                   CHECK_TOPOLOGY added to ecat setup sequence
'
'      28.1.2016  V1.0.0.6 - Use constant for BUS identification: CAN and EtherCAT
'
'      12.4.2016  V1.0.0.7 - Adapted to work with AXLFBKEC.LIB instead of the deprecated EC_AI8ME.LIB
'
'**********************************************************************************************


'**********************************************************************************************

'                                      TO DO !
' Once it is possible to change the slaves ECat opmode from MC-Basic, invoke EC_STARTMASTER after
' call EC_SET_CYCLETIME(user_cycletime) and before the For loop that sets the encoder resolution
' in each drive. In order to do so, the drives must be put to PREOP, so uncomment the execution
' of call EC_Slave_NMT_StateSet in the beginning and the end of the For loop.

'**********************************************************************************************


' module global "constants"

' module global variables


common shared numOfSlaves as long = 0
common shared num_Of_Motion_Drives as long = 0

dim shared num_Of_Digital_Inputs as long = 4
dim shared num_Of_Digital_Outputs as long = 4

dim shared Note_PLL_Error_Retry as note "PLL Error in the drives. Restarting EC_SETUP.PRG" 20300

Program 

  dim Axis_Attached_To_Drive as long = 1
  dim Counts_Per_Revolution as long = 10000  '  Set to 10000 by default for a generic drive
  dim ec_setup_iterations as long = 1
  dim retVal as long = 0
  dim drive_addr as long = 0
  dim PLL_Error as long = 0


START_EC_SETUP_PRG:

  numOfSlaves = 0
  num_Of_Motion_Drives = 0
  Axis_Attached_To_Drive = 1
  Counts_Per_Revolution = 10000  '  Set to 10000 by default for a generic drive
  retVal = 0
  drive_addr = 0
  PLL_Error = 0

  numOfSlaves = EC_ETHERCAT_INIT ' Initialize EtherCAT according to a certain field bus, and type of slave addressing

  ' In CDHD motion drives, remap PDOS according to softMC requirements
  for drive_addr = 1 to numOfSlaves

    if ETSLAVES[drive_addr]->et_slavetype = ECAT_MOTIONSLAVE then

      if EC_SLAVES_INFO[drive_addr]->vendor_id = SERVOTRONIX_VENDOR_ID_0X2E1 then

        if EC_SLAVES_INFO[drive_addr]->product_code = STX_CDHD_PRODUCT_CODE_0X0 then
            if EC_SLAVES_INFO[drive_addr]->product_code = STX_CDHD_PRODUCT_CODE_0X1 then

              call EC_REMAP_CDHD_DEFAULT_PDOS(drive_addr)
              
            end if  

        end if

      end if

    end if

  next

  retVal = EC_RESCAN_SLAVES
  sleep 500

  ' Check the EtherCAT topology for miswiring
  retVal = CHECK_TOPOLOGY  '  throws error

  call EC_CREATE_MASTER

  print "Found ";numOfSlaves;" slaves"

  call EC_SET_BUS_CYCLETIME(4000)  '  For a cycle time of 500 us, add to FWCONFIG: hpetfreq = 2000

  EC_ADDRESS_BY_ALIAS = 0 ' Set to zero -> alias addressing is position based


  '****************************************************************
  '                                *
  '        PRE - OPERATIONAL CONFIGURATION          *
  ' Configure each slave according to Vendor ID and Product Code  *
  '                                *
  '****************************************************************

  for drive_addr = 1 to numOfSlaves

    ' Motion Drives
    Select Case EC_SLAVES_INFO[drive_addr]->vendor_id

      Case SERVOTRONIX_VENDOR_ID_0X2E1, PBA_SYS_MAXTUNE_VENDOR_ID, GOOGOLTECH_GTHD_VENDOR_ID, MPC_FPRO_VENDOR_ID, AKRIBIS_ASD_VENDOR_ID, TECO_JSDZ_VENDOR_ID, HIGEN_SDA7_VENDOR_ID, SCHNEIDER_VENDOR_ID

        if EC_SLAVES_INFO[drive_addr]->product_code = STX_CDHD_PRODUCT_CODE_0X0 OR EC_SLAVES_INFO[drive_addr]->product_code = STX_CDHD_PRODUCT_CODE_0X1 then  '  CDHD Motion Drive

          call EC_INSTALL_STX_CDHD(drive_addr, EC_SLAVES_INFO[drive_addr]->vendor_id, EC_SLAVES_INFO[drive_addr]->product_code, CDHD_OPMODE_SYNC_POSITION, 2)  '  mic_int = 2
          num_Of_Motion_Drives = num_Of_Motion_Drives + 1

        end if  '  STX_CDHD_PRODUCT_CODE_0X0


      Case GENERIC_DS402_DRIVE_VENDOR  '  replace with your vendor id

        if EC_SLAVES_INFO[drive_addr]->product_code = GENERIC_DS402_DRIVE_PRODUCT_ID then  '  replace with your product id

          call EC_INSTALL_GENERIC_DRIVE(drive_addr, Counts_Per_Revolution)  '  Counts Per motor Revolution
          num_Of_Motion_Drives = num_Of_Motion_Drives + 1

        end if


      ' IO Modules
      Case ESD_VENDOR_ID_0X17

        if EC_SLAVES_INFO[drive_addr]->product_code = ECX_DIO8_PRODUCT_CODE_0X4 then  '  ESD - ECX_DIO8

          call EC_INSTALL_ECX_DIO8(drive_addr)

        end if


      Case PHOENIX_CON_VENDOR_ID_0X84

        if EC_SLAVES_INFO[drive_addr]->product_code = AXLFBKEC_PRODUCT_CODE_0X290783 then  '  AXL F BK EC  -  Bus Coupler

          ' Input data into AXL_SLAVE_INFO array
          AXL_Slices[1]->PDO_index = 0x6000
          AXL_Slices[1]->PDO_subindex = 0x1
          AXL_Slices[1]->num_of_bits = 16
          AXL_Slices[1]->first_sys_dio = 700
          call EC_INSTALL_AXL_F_BK_EC(drive_addr, AXL_Slices, 1)  '  1 - Number of slices. Change according to real setup

        end if


      Case FESTO_VENDOR_ID_0X1D

        if EC_SLAVES_INFO[drive_addr]->product_code = PCX_FB38_PRODUCT_CODE_0X26 then

          call EC_INSTALL_FESTO_CPX_FB38(drive_addr)

        end if

        if EC_SLAVES_INFO[drive_addr]->product_code = CTEU_PRODUCT_CODE_0x8bc8c then

          call EC_INSTALL_FESTO_CTEU_0x8bc8c(drive_addr)

        end if


      Case HCNC_VENDOR_ID_0X353

        if EC_SLAVES_INFO[drive_addr]->product_code = HCNC_HIO_1065_PRODUCT_CODE then

          call EC_INSTALL_HCNC_HIO_1065(drive_addr)
          drive_addr = drive_addr + 2  '  Advance over 2 slices of inputs and outputs

        end if


      Case BECKHOFF_VENDOR_ID_0X2

        if EC_SLAVES_INFO[drive_addr]->product_code = EK1100_PRODUCT_CODE then

          'call EC_INSTALL_BECKHOFF_EK1100(drive_addr)

        end if

        if EC_SLAVES_INFO[drive_addr]->product_code = EL3356_PRODUCT_CODE then

          call EC_INSTALL_BECKHOFF_EL3356(drive_addr)

        end if

        if EC_SLAVES_INFO[drive_addr]->product_code = EL6752_PRODUCT_CODE then

          call EC_INSTALL_BECKHOFF_EL6752(drive_addr)

        end if

        if EC_SLAVES_INFO[drive_addr]->product_code = EL1008_PRODUCT_CODE then

          call EC_INSTALL_BECKHOFF_EL1008(drive_addr)

        end if

        if EC_SLAVES_INFO[drive_addr]->product_code = EL1088_PRODUCT_CODE then

          call EC_INSTALL_BECKHOFF_EL1088(drive_addr)

        end if

        if EC_SLAVES_INFO[drive_addr]->product_code = EL2088_PRODUCT_CODE then

          call EC_INSTALL_BECKHOFF_EL2088(drive_addr)

        end if


      Case HMS_VENDOR_ID_0X1B

        if EC_SLAVES_INFO[drive_addr]->product_code = X_GATEWAY_PRODUCT_CODE then

          call EC_INSTALL_X_GATEWAY(drive_addr)

        end if


      Case WAGO_VENDOR_ID_0X21

        if EC_SLAVES_INFO[drive_addr]->product_code = FC750_PRODUCT_CODE then

          call EC_INSTALL_WAGO_FC750(drive_addr)

        end if


      Case SAFETY_VENDOR_ID_0X569

        if EC_SLAVES_INFO[drive_addr]->product_code = SAFETY_PRODUCT_CODE then

          call EC_INSTALL_SAFETY(drive_addr)

        end if


      Case WEIDMULLER_VENDOR_ID_0X230

        if EC_SLAVES_INFO[drive_addr]->product_code = UR20_FBC_EC_PRODUCT_CODE then

          call EC_INSTALL_WEIDMULLER_UR20_FBC(drive_addr)

        end if


    End Select


    '****************************************************************
    '                                *
    '           USER PREOP CONFIGURATION            *
    ' Add code to EC_USER_PREOP_CONFIG(byval drive_addr as long)  *
    '                                *
    '****************************************************************
    retVal = EC_USER_PREOP_CONFIG(drive_addr)


  next


  ' Create PDOs mapping, set bus cycle time, sync clocks between the master and the slaves,
  ' raise slaves to OP-Mode and sets master's opmode to operational
  Try

    call EC_STARTMASTER

  Catch EC_MASTER_PLL_ERROR.num

    PLL_Error = 1

  End Try

  ' In case of PLL error in the drives we make up to 5 attempts to restart EC_SETUP
  if PLL_Error = 1 then

    if ec_setup_iterations < 6 then

      ec_setup_iterations = ec_setup_iterations + 1
      print "PLL Error in the drives. Restarting EC_SETUP.PRG attempt #: ";ec_setup_iterations
      Logger Note_PLL_Error_Retry
      call ec_stopmaster
      sleep 3000
      goto START_EC_SETUP_PRG

    else

      call ec_stopmaster
      throw EC_MASTER_PLL_ERROR

    end if

  end if

  '****************************************************************
  '                                *
  '           OPERATIONAL CONFIGURATION          *
  ' Configure each slave according to Vendor ID and Product Code  *
  '                                *
  '****************************************************************

  for drive_addr = 1 to numOfSlaves

    ' SAFEOP state configuration - Add code to EC_USER_SAFEOP_CONFIG(byval drive_addr as long)
    retVal = EC_USER_SAFEOP_CONFIG(drive_addr)

    ' OP state configuration - Add code to EC_USER_OP_CONFIG(byval drive_addr as long)
    retVal = EC_USER_OP_CONFIG(drive_addr)

    if ETSLAVES[drive_addr]->et_slavetype = ECAT_MOTIONSLAVE then

      ' Assign slave address to each axis
      systemAxis(Axis_Attached_To_Drive).busnumber = ECAT_BUSID
      systemAxis(Axis_Attached_To_Drive).dadd = drive_addr

      ' Attempt to clear faults in the drives
      Try

        call EC_CLEAR_FAULTS(drive_addr)

      Catch Else

        ' if clear faults failed due to PLL Sync Error, wait a little longer and retry
        if EC_SDO_READ(drive_addr, 0x603f, 0) = 0x7386 then
          printu "PLL Error in drive #. Sleeping 5 seconds and retrying to clear fault";drive_addr
          sleep 5000
          call EC_CLEAR_FAULTS(drive_addr)
        end if

      End Try

      ' Set drives to sync position mode
      if EC_IS_PDO(systemAxis(Axis_Attached_To_Drive).dadd, 0x6060, 0) then
        call EC_PDO_WRITE(systemAxis(Axis_Attached_To_Drive).dadd, Modes_Of_Operation_6060h, 0, 8)
      else
        call EC_SDO_WRITE(systemAxis(Axis_Attached_To_Drive).dadd, Modes_Of_Operation_6060h, 0, 8, 8)
      end if

      Select Case EC_SLAVES_INFO[drive_addr]->vendor_id

        Case SERVOTRONIX_VENDOR_ID_0X2E1

          if EC_SLAVES_INFO[drive_addr]->product_code = STX_CDHD_PRODUCT_CODE_0X0 OR EC_SLAVES_INFO[drive_addr]->product_code = STX_CDHD_PRODUCT_CODE_0X1 then  '  CDHD Motion Drive

            ' Add additional code if required

          end if  '  STX_CDHD_PRODUCT_CODE_0X0


        Case GENERIC_DS402_DRIVE_VENDOR

          if EC_SLAVES_INFO[drive_addr]->product_code = GENERIC_DS402_DRIVE_PRODUCT_ID then

            ' Add additional code if required

          end if


      End Select

      Axis_Attached_To_Drive = Axis_Attached_To_Drive + 1

    end if  '  ECAT_MOTIONSLAVE


  next

  print "EtherCAT Setup Finished Successfully !"

End Program




