'**********************************************************************************************
' File:			REG_ERR.PRG
' Purpose:		Identify EtherCAT communication errors
' Version:		1.00
' Author:		Nir Geller
' Description:	By reading registers from the drives the script will try to identify the origin 
'				of bus faults and Ecat errors.
'
' History:		1.11.2014  V1.00
'**********************************************************************************************

'****************************************************************************************************
'
'  CDHD drives have only port 0 and port 1 (C5 and C6). There are EtherCAT slave controllers
'  that additionally have port 2 and port 3. The code is included in the script but it is commented out.
'
'****************************************************************************************************


common shared ESC_DL_Stat_Reg_1 as long = 0x0110
common shared ESC_DL_Stat_Reg_2 as long = 0x0111


common shared CRC_Error_Counter_Port_0_Reg as long = 0x0300
common shared Phy_Error_Counter_Port_0_Reg as long = 0x0301
common shared CRC_Error_Counter_Port_1_Reg as long = 0x0302
common shared Phy_Error_Counter_Port_1_Reg as long = 0x0303
'common shared CRC_Error_Counter_Port_2_Reg as long = 0x0304
'common shared Phy_Error_Counter_Port_2_Reg as long = 0x0305
'common shared CRC_Error_Counter_Port_3_Reg as long = 0x0306
'common shared Phy_Error_Counter_Port_3_Reg as long = 0x0307


common shared Fwd_CRC_Error_Counter_Port_0_Reg as long = 0x0308
common shared Fwd_CRC_Error_Counter_Port_1_Reg as long = 0x0309
'common shared Fwd_CRC_Error_Counter_Port_2_Reg as long = 0x030A
'common shared Fwd_CRC_Error_Counter_Port_3_Reg as long = 0x030B


common shared Lost_Link_Counter_Port_0_Reg as long = 0x0310
common shared Lost_Link_Counter_Port_1_Reg as long = 0x0311
'common shared Lost_Link_Counter_Port_2_Reg as long = 0x0312
'common shared Lost_Link_Counter_Port_3_Reg as long = 0x0313





program

	dim base_address as long = 0x0300
	dim retVal as long = 0
	dim i as long = 0
	dim drive_address as long = 1
	dim Bit_Mask as long = 0x4

	print ""
	print "Reading Registers:"

	for drive_address = 1 to ec_num_of_slaves  '  Should iterate only over motion drives

		if ETSLAVES[drive_address]->et_slavetype = ECAT_IOSLAVE then
			goto CONTINUE_FOR_LOOP
		end if

		print "----------------------------------------------------------------------------"


		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, ESC_DL_Stat_Reg_1, 1)
		Bit_Mask = 2^4
		if (retVal BAND Bit_Mask) = 0 then  '  No link detected
			print "Slave ";drive_address;": Register 0x0110: No Link Detected on Port 0"
		end if
		Bit_Mask = Bit_Mask SHL 1
		if (retVal BAND Bit_Mask) = 0 then  '  No link detected
			print "Slave ";drive_address;": Register 0x0110: No Link Detected on Port 1"
		end if
		'Bit_Mask = Bit_Mask SHL 1
		'if (retVal BAND Bit_Mask) = 0 then  '  No link detected
		'  print "Slave ";drive_address;": Register 0x0110: No Link Detected on Port 2"
		'end if
		'Bit_Mask = Bit_Mask SHL 1
		'if (retVal BAND Bit_Mask) = 0 then  '  No link detected
		'  print "Slave ";drive_address;": Register 0x0110: No Link Detected on Port 3"
		'end if


		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, ESC_DL_Stat_Reg_2, 1)
		if retVal BAND 2^0 then  '  Port Closed
			print "Slave ";drive_address;": Register 0x0111: " + "Port 0 is Closed"
		end if
		if retVal BAND 2^2 then
			print "Slave ";drive_address;": Register 0x0111: " + "Port 1 is Closed"
		end if
		'if retVal BAND 2^4 then
		'  print "Slave ";drive_address;": Register 0x0111: " + "Port 2 is Closed"
		'end if
		'if retVal BAND 2^6 then
		'  print "Slave ";drive_address;": Register 0x0111: " + "Port 3 is Closed"
		'end if


		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, CRC_Error_Counter_Port_0_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0300: CRC Error Counter Port 0	= ";(retVal BAND 0xFF) hex
		end if

		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, Phy_Error_Counter_Port_0_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0301: Physical Interface Error Counter Port 0	= ";(retVal BAND 0xFF) hex
		end if

		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, CRC_Error_Counter_Port_1_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0302: CRC Error Counter Port 1	= ";(retVal BAND 0xFF) hex
		end if

		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, Phy_Error_Counter_Port_1_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0303: Physical Interface Error Counter Port 1	= ";(retVal BAND 0xFF) hex
		end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, CRC_Error_Counter_Port_2_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x0304: CRC Error Counter Port 2	= ";(retVal BAND 0xFF) hex
		'end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, Phy_Error_Counter_Port_2_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x0305: Physical Interface Error Counter Port 2	= ";(retVal BAND 0xFF) hex
		'end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, CRC_Error_Counter_Port_3_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x0306: CRC Error Counter Port 3	= ";(retVal BAND 0xFF) hex
		'end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, Phy_Error_Counter_Port_3_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x0307: Physical Interface Error Counter Port 3	= ";(retVal BAND 0xFF) hex
		'end if


		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, Fwd_CRC_Error_Counter_Port_0_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0308: Forwarded CRC Error Counter Port 0	= ";(retVal BAND 0xFF) hex
		end if

		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, Fwd_CRC_Error_Counter_Port_1_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0309: Forwarded CRC Error Counter Port 1	= ";(retVal BAND 0xFF) hex
		end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, Fwd_CRC_Error_Counter_Port_2_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x030A: Forwarded CRC Error Counter Port 2	= ";(retVal BAND 0xFF) hex
		'end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, Fwd_CRC_Error_Counter_Port_3_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x030B: Forwarded CRC Error Counter Port 3	= ";(retVal BAND 0xFF) hex
		'end if


		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, Lost_Link_Counter_Port_0_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0310: Lost Link Counter Port 0					= ";(retVal BAND 0xFF) hex
		end if

		retVal = 0
		retVal = EC_SLAVE_READ_REGISTER(drive_address, Lost_Link_Counter_Port_1_Reg, 1)
		if (retVal BAND 0xFF) <> 0 then
			print "Slave ";drive_address;": Register 0x0311: Lost Link Counter Port 1					= ";(retVal BAND 0xFF) hex
		end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, Lost_Link_Counter_Port_2_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x0312: Lost Link Counter Port 2					= ";(retVal BAND 0xFF) hex
		'end if

		'retVal = 0
		'retVal = EC_SLAVE_READ_REGISTER(drive_address, Lost_Link_Counter_Port_3_Reg, 1)
		'if (retVal BAND 0xFF) <> 0 then
		'  print "Slave ";drive_address;": Register 0x0313: Lost Link Counter Port 3					= ";(retVal BAND 0xFF) hex
		'end if


CONTINUE_FOR_LOOP:

	next

end program
